"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const body_parser_1 = __importDefault(require("body-parser"));
const express_1 = __importDefault(require("express"));
const fs_1 = __importDefault(require("fs"));
const asap_request_1 = __importDefault(require("./asap_request"));
const autoscale_poller_1 = __importDefault(require("./autoscale_poller"));
const command_handler_1 = __importDefault(require("./command_handler"));
const config_1 = __importDefault(require("./config"));
const logger_1 = __importDefault(require("./logger"));
const stats_reporter_1 = __importDefault(require("./stats_reporter"));
const jwtSigningKey = fs_1.default.readFileSync(config_1.default.AsapSigningKeyFile);
const metadata = config_1.default.InstanceMetadata;
const instanceDetails = {
    instanceId: config_1.default.InstanceId,
    instanceType: config_1.default.InstanceType,
    ...metadata
};
const commandHandler = new command_handler_1.default({
    gracefulScript: config_1.default.GracefulShutdownScript,
    terminateScript: config_1.default.TerminateScript,
    reconfigureScript: config_1.default.ReconfigureScript
});
const asapRequest = new asap_request_1.default({
    signingKey: jwtSigningKey,
    asapJwtIss: config_1.default.AsapJwtIss,
    asapJwtAud: config_1.default.AsapJwtAud,
    asapJwtKid: config_1.default.AsapJwtKid
});
const autoscalePoller = new autoscale_poller_1.default({
    pollUrl: config_1.default.PollingURL,
    statusUrl: config_1.default.StatusURL,
    statsUrl: config_1.default.StatsReportURL,
    instanceDetails,
    asapRequest
});
const statsReporter = new stats_reporter_1.default({
    retrieveUrl: config_1.default.StatsRetrieveURL,
    asapRequest,
    instanceDetails
});
logger_1.default.info('Starting up sidecar with config', { config: config_1.default });
// loop for polling, reporting stats when available
let statsReport;
let reconfigureLock = false;
let shutdownLock = false;
/**
 * The hook to give jibri state.
 *
 * @param req the request.
 * @param res the response.
 */
async function jibriStateWebhook(req, res) {
    const instate = req.body;
    if (!instate.status) {
        res.sendStatus(400);
        return;
    }
    if (!instate.jibriId) {
        res.sendStatus(400);
        return;
    }
    // update global stats report with
    statsReport = statsReporter.buildStatsReport(instate);
    await autoscalePoller.reportStats(statsReport);
    res.status(200);
    res.send('{"status":"OK"}');
}
/**
 * Polls stats every 1 second.
 */
async function pollForStats() {
    try {
        statsReport = await statsReporter.retrieveStatsReport();
    }
    catch (err) {
        logger_1.default.error('Error collecting stats', { err });
        statsReport = undefined;
    }
    setTimeout(pollForStats, config_1.default.StatsPollingInterval * 1000);
}
/**
 * Poll for status every second and checks for reconfigure or shutdown result.
 */
async function pollForStatus() {
    let pollResult;
    // poll for shutdown/reconfigure, optionally sending stats if available
    try {
        pollResult = await autoscalePoller.pollWithStats(statsReport);
    }
    catch (err) {
        logger_1.default.error('Polling failed', { err });
    }
    // trigger next tick of polling
    setTimeout(pollForStatus, config_1.default.ShutdownPollingInterval * 1000);
    // handle poll results, optionally shutting down or starting reconfigure
    if (pollResult.shutdown) {
        logger_1.default.info('Shutdown detected, triggering shutdown');
        if (shutdownLock) {
            logger_1.default.info('Shutdown already detected, skipping shutdown trigger');
        }
        else {
            shutdownLock = true;
            statsReporter.setShutdownStatus(true);
            // shutting down, so don't do anything else
            try {
                await commandHandler.shutdown();
                statsReporter.setShutdownError(false);
            }
            catch (err) {
                statsReporter.setShutdownError(true);
                // unlock shutdown only if error occurred
                shutdownLock = false;
            }
        }
    }
    else if (pollResult.reconfigure) {
        if (reconfigureLock) {
            logger_1.default.info('Reconfigure already running, skipping reconfigure trigger');
        }
        else {
            logger_1.default.info('Reconfigure detected, triggering reconfiguration');
            // set reconfigure lock, only reconfigure if configure is not already running
            reconfigureLock = true;
            try {
                statsReporter.setReconfigureStart(pollResult.reconfigure);
                await commandHandler.reconfigure();
                statsReporter.setReconfigureEnd(false);
                logger_1.default.info('Reconfiguration completed');
            }
            catch (err) {
                statsReporter.setReconfigureEnd(true);
                logger_1.default.error('Reconfiguration failed', { err });
            }
            // reconfigure is done
            reconfigureLock = false;
        }
    }
}
if (config_1.default.EnableReportStats) {
    pollForStats().then(() => {
        pollForStatus();
    });
}
else {
    pollForStatus();
}
const app = express_1.default();
app.use(body_parser_1.default.json());
app.get('/health', (req, res) => {
    res.send('healthy!');
});
app.post('/hook/v1/status', async (req, res, next) => {
    try {
        await jibriStateWebhook(req, res);
    }
    catch (err) {
        next(err);
    }
});
app.listen(config_1.default.HTTPServerPort, () => {
    logger_1.default.info(`...listening on :${config_1.default.HTTPServerPort}`);
});
//# sourceMappingURL=app.js.map