"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const got_1 = __importDefault(require("got"));
const jsonwebtoken_1 = require("jsonwebtoken");
const node_cache_1 = __importDefault(require("node-cache"));
/**
 * The asap request.
 */
class AsapRequest {
    /**
     * Constructs request.
     * @param options
     */
    constructor(options) {
        this.cacheTTL = 60 * 45;
        this.requestTimeout = 3 * 1000;
        this.requestRetryCount = 2;
        this.signingKey = options.signingKey;
        this.asapJwtIss = options.asapJwtIss;
        this.asapJwtAud = options.asapJwtAud;
        this.asapJwtKid = options.asapJwtKid;
        if (options.requestTimeout !== undefined) {
            this.requestTimeout = options.requestTimeout;
        }
        if (options.requestRetryCount !== undefined) {
            this.requestRetryCount = options.requestRetryCount;
        }
        if (options.cacheTTL !== undefined) {
            this.cacheTTL = options.cacheTTL;
        }
        this.asapCache = new node_cache_1.default({ stdTTL: this.cacheTTL }); // TTL of 45 minutes
        this.authToken = this.authToken.bind(this);
        this.postJson = this.postJson.bind(this);
        this.getJson = this.getJson.bind(this);
    }
    /**
     * Returns an auth token.
     */
    authToken() {
        const cachedAuth = this.asapCache.get('asap');
        if (cachedAuth) {
            return cachedAuth;
        }
        const auth = jsonwebtoken_1.sign({}, this.signingKey, {
            issuer: this.asapJwtIss,
            audience: this.asapJwtAud,
            algorithm: 'RS256',
            keyid: this.asapJwtKid,
            expiresIn: 60 * 60 // 1 hour
        });
        this.asapCache.set('asap', auth);
        return auth;
    }
    /**
     * Posts a json to the specified url.
     * @param url the url.
     * @param body the body to add.
     */
    async postJson(url, body) {
        const response = await got_1.default.post(url, {
            headers: {
                Authorization: `Bearer ${this.authToken()}`
            },
            json: body,
            responseType: 'json',
            timeout: this.requestTimeout,
            retry: this.requestRetryCount
        });
        return response.body;
    }
    /**
     * Gets a json from url.
     * @param url the url to use.
     */
    async getJson(url) {
        const response = await got_1.default.get(url, {
            headers: {
                Authorization: `Bearer ${this.authToken()}`
            },
            responseType: 'json',
            timeout: this.requestTimeout,
            retry: this.requestRetryCount
        });
        return response.body;
    }
}
exports.default = AsapRequest;
//# sourceMappingURL=asap_request.js.map