"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const util_1 = require("util");
const logger_1 = __importDefault(require("./logger"));
const pexec = util_1.promisify(child_process_1.exec);
/**
 * Handles commands.
 */
class CommandHandler {
    /**
     * Constructs the handler.
     * @param options the options to use.
     */
    constructor(options) {
        this.gracefulScript = options.gracefulScript;
        this.terminateScript = options.terminateScript;
        this.reconfigureScript = options.reconfigureScript;
        this.shutdown = this.shutdown.bind(this);
        this.gracefulShutdown = this.gracefulShutdown.bind(this);
        this.reconfigure = this.reconfigure.bind(this);
    }
    /**
     * Executes graceful shutdown.
     */
    async gracefulShutdown() {
        logger_1.default.info('Handling graceful shutdown', {
            gracefulScript: this.gracefulScript
        });
        let result;
        try {
            result = await pexec(this.gracefulScript);
            logger_1.default.info(result.stdout);
            logger_1.default.info(result.stderr);
        }
        catch (err) {
            logger_1.default.error('shutdown error', { err,
                stdout: result.stdout,
                stderr: result.stderr });
            throw err;
        }
        return true;
    }
    /**
     * Executes reconfigure.
     */
    async reconfigure() {
        logger_1.default.info('Handling reconfigure', {
            reconfigureScript: this.reconfigureScript
        });
        let result;
        try {
            result = await pexec(this.reconfigureScript);
            logger_1.default.info(result.stdout);
            logger_1.default.info(result.stderr);
        }
        catch (err) {
            logger_1.default.error('reconfiguration error', { err,
                stdout: result.stdout,
                stderr: result.stderr });
            throw err;
        }
        return true;
    }
    /**
     * Executes terminate the instance.
     */
    async terminateInstance() {
        logger_1.default.info('Handling instance termination', {
            terminateScript: this.terminateScript
        });
        let result;
        try {
            const { stdout, stderr } = await pexec(this.terminateScript);
            logger_1.default.info(stdout);
            logger_1.default.info(stderr);
        }
        catch (err) {
            logger_1.default.error('terminate error', { err,
                stdout: result.stdout,
                stderr: result.stderr });
            throw err;
        }
        return true;
    }
    /**
     * Executes shutdown.
     */
    async shutdown() {
        logger_1.default.info('Handling shutdown', {
            gracefulScript: this.gracefulScript,
            terminateScript: this.terminateScript
        });
        try {
            const result = await this.gracefulShutdown();
            if (result) {
                logger_1.default.info('Successful graceful shutdown, terminating');
            }
            else {
                logger_1.default.error('Failed graceful shutdown, terminating');
            }
            await this.terminateInstance();
        }
        catch (err) {
            logger_1.default.error('shutdown error detected, forcing termination');
            await this.terminateInstance();
            throw err;
        }
        return true;
    }
}
exports.default = CommandHandler;
//# sourceMappingURL=command_handler.js.map