"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = __importDefault(require("./logger"));
/**
 * The stats reporter.
 */
class StatsReporter {
    /**
     * Constructs the reporter.
     * @param options the options to use.
     */
    constructor(options) {
        this.retrieveUrl = options.retrieveUrl;
        this.asapRequest = options.asapRequest;
        this.instanceDetails = options.instanceDetails;
        this.shutdownStatus = false;
        this.shutdownError = false;
        this.reconfigureError = false;
        this.reconfigureComplete = '';
        this.statsError = false;
        this.retrieveStats = this.retrieveStats.bind(this);
        this.setShutdownStatus = this.setShutdownStatus.bind(this);
        this.retrieveStatsReport = this.retrieveStatsReport.bind(this);
    }
    /**
     * Sets shutdown status.
     * @param status the new value.
     */
    setShutdownStatus(status) {
        this.shutdownStatus = status;
    }
    /**
     * Sets shutdown error.
     * @param shutdownError the new value.
     */
    setShutdownError(shutdownError) {
        this.shutdownError = shutdownError;
    }
    /**
     * Sets reconfigure start value.
     * @param reconfigureStarted the new value.
     */
    setReconfigureStart(reconfigureStarted) {
        // initialize the reconfiguration error handler and start time
        this.reconfigureStarted = reconfigureStarted;
        this.reconfigureError = false;
    }
    /**
     * Sets reconfigure end value.
     * @param reconfigureError the new value.
     */
    setReconfigureEnd(reconfigureError) {
        this.reconfigureError = reconfigureError;
        this.reconfigureComplete = this.reconfigureStarted;
        this.reconfigureStarted = undefined;
    }
    /**
     * Sets stats error.
     * @param status the new value.
     */
    setStatsError(status) {
        this.statsError = status;
    }
    /**
     * Retrieves the stats json.
     */
    async retrieveStats() {
        try {
            const response = await this.asapRequest.getJson(this.retrieveUrl);
            if (response) {
                return response;
            }
        }
        catch (err) {
            logger_1.default.error('RetrieveStats Error', { err,
                url: this.retrieveUrl });
        }
    }
    /**
     * Retrieves the stats report from the json.
     */
    async retrieveStatsReport() {
        let stats = await this.retrieveStats();
        if (stats) {
            // clear any previous stats errors that may have been raised
            this.setStatsError(false);
        }
        else {
            logger_1.default.error('Empty stats, error occurred, returning blank report');
            this.setStatsError(true);
            stats = {};
        }
        const report = this.buildStatsReport(stats);
        return report;
    }
    /**
     * Build a stats report.
     * @param stats the stats to use.
     */
    buildStatsReport(stats) {
        const ts = new Date();
        const report = {
            instance: this.instanceDetails,
            stats,
            timestamp: ts.getTime(),
            shutdownStatus: this.shutdownStatus,
            shutdownError: this.shutdownError,
            reconfigureError: this.reconfigureError,
            reconfigureComplete: this.reconfigureComplete,
            statsError: this.statsError
        };
        logger_1.default.debug('Stats report', { report });
        return report;
    }
}
exports.default = StatsReporter;
//# sourceMappingURL=stats_reporter.js.map